@@ -6,7 +6,7 @@
 ############################################################################################
 
 LLsurface.secr <- function (object, betapar = c('g0', 'sigma'), xval = NULL, yval = NULL,
-    centre = NULL, realscale = TRUE, plot = TRUE, plotfitted = TRUE, ...) {
+    centre = NULL, realscale = TRUE, plot = TRUE, plotfitted = TRUE, ncores = 1, ...) {
 
     if (inherits(object, 'list')) {
         temp <- list()
@@ -67,12 +67,25 @@ LLsurface.secr <- function (object, betapar = c('g0', 'sigma'), xval = NULL, yva
                 = object$fixed, timecov = object$timecov, sessioncov =
                 object$sessioncov, groups = object$groups, dframe =
                 object$dframe, details = details, method =
-                object$fit$method, verify = FALSE) )
+                object$fit$method, verify = FALSE, ncores = 1)
+                )
         }
 
         cat ('Evaluating log likelihood across grid of', nrow(grid), 'points...\n')
         flush.console()
-        temp <- apply (grid, 1, LL)
+
+        if (ncores > 1) {
+            require(parallel)
+            clust <- makeCluster(ncores, methods = FALSE, useXDR = FALSE)
+            clusterEvalQ(clust, library(secr))
+            clusterExport(clust, c("object", "details"), environment())
+            temp <- parRapply (clust, grid, LL)
+            stopCluster(clust)
+        }
+        else {
+            temp <- apply (grid, 1, LL)
+        }
+
         temp <- matrix(temp, nrow=length(xval))
         if (realscale) {
             xval <- round(untransform(xval, linkx),4)
