@@ -107,13 +107,14 @@ autoini <- function (capthist, mask, detectfn = 0, thin = 0.2, tol = 0.001,
           as.integer(dettype),
           as.integer(param),
           as.double(g0sigma0),
-          as.integer(rep(1,nc)),        # group number 2012-11-13
+          as.integer(rep(0,nc)),        # group number 2012-11-13; 2013-04-16
           as.integer(nc),
           as.integer(s),
           as.integer(k),
           as.integer(m),
           as.integer(1),                # number of groups 2012-11-13
           as.integer(1),
+          as.integer(rep(1,nc)),        # knownclass 2013-04-12, 2013-06-04
           as.double(unlist(trps)),
           as.double(unlist(usge)),
           as.double(unlist(mask)),
